# Copyright (C) 2017-2020  The Project X-Ray Authors.
#
# Use of this source code is governed by a ISC-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/ISC
#
# SPDX-License-Identifier: ISC
# XC7010-1CLG400C
export XRAY_PINCFG=ZYBOZ7-SWBUT
export XRAY_DIN_N_LARGE=4
export XRAY_DOUT_N_LARGE=6
export HARNESS_DIR=$XRAY_DIR/database/zynq7/harness/zybo/swbut/

source $XRAY_DIR/minitests/roi_harness/zybo-common.sh
