#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017-2020  The Project X-Ray Authors.
#
# Use of this source code is governed by a ISC-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/ISC
#
# SPDX-License-Identifier: ISC

ports = {
    "DSP48E1": [
        ("ALUMODE", 4),
        ("CARRYINSEL", 3),
        ("CEAD", 1),
        ("CEALUMODE", 1),
        ("CED", 1),
        ("CEINMODE", 1),
        ("D", 25),
        ("INMODE", 5),
        ("OPMODE", 7),
        ("RSTD", 1),
    ],
}
