#!/usr/bin/env bash
# Copyright (C) 2017-2020  The Project X-Ray Authors.
#
# Use of this source code is governed by a ISC-style
# license that can be found in the LICENSE file or at
# https://opensource.org/licenses/ISC
#
# SPDX-License-Identifier: ISC

touch build/database/${XRAY_DATABASE}/segbits_clblm_l.db
touch build/database/${XRAY_DATABASE}/segbits_clblm_r.db
touch build/database/${XRAY_DATABASE}/segbits_clbll_l.db
touch build/database/${XRAY_DATABASE}/segbits_clbll_r.db
