/*
 * Copyright (C) 2017-2020  The Project X-Ray Authors.
 *
 * Use of this source code is governed by a ISC-style
 * license that can be found in the LICENSE file or at
 * https://opensource.org/licenses/ISC
 *
 * SPDX-License-Identifier: ISC
 */
#ifndef PRJXRAY_LIB_XILINX_NOP_PACKET_H
#define PRJXRAY_LIB_XILINX_NOP_PACKET_H

#include <prjxray/xilinx/configuration_packet.h>
#include <prjxray/xilinx/configuration_register.h>

namespace prjxray {
namespace xilinx {

template <typename ConfigRegType>
class NopPacket : public ConfigurationPacket<ConfigRegType> {
       public:
	NopPacket()
	    : ConfigurationPacket<ConfigRegType>(
	          ConfigurationPacketType::TYPE1,
	          ConfigurationPacket<ConfigRegType>::Opcode::NOP,
	          ConfigRegType::CRC,
	          {}) {}
};

}  // namespace xilinx
}  // namespace prjxray

#endif  // PRJXRAY_LIB_XILINX_NOP_PACKET_H
